@@ -1,6 +1,3 @@
-import functools
-import types
-
 import torch._C as _C
 
 TensorProtoDataType = _C._onnx.TensorProtoDataType
@@ -19,7 +16,8 @@ class ExportTypes:
 
 def _export(*args, **kwargs):
     from torch.onnx import utils
-    return utils._export(*args, **kwargs)
+    result = utils._export(*args, **kwargs)
+    return result
 
 
 def export(*args, **kwargs):
@@ -55,3 +53,8 @@ def _run_symbolic_function(*args, **kwargs):
 def _run_symbolic_method(*args, **kwargs):
     from torch.onnx import utils
     return utils._run_symbolic_method(*args, **kwargs)
+
+
+def is_in_onnx_export():
+    from torch.onnx import utils
+    return utils.is_in_onnx_export()
